/**
 * A submission description.
 */
export default class Submission {
    /**
     * Creates a new submission.
     *
     * @param aid the identifier of the assignment the submission was for
     * @param id the identifier
     * @param name the name
     * @param {String} identity the identity string
     * @param $client the user's AuTA client
     */
    constructor(aid, id, name, identity, $client) {
        this.aid = aid;
        this.id = id;
        this.name = name;
        this.identity = identity;
        this.$client = $client;
    }

    /**
     * Returns the submission's verdict.
     *
     * @returns {Promise<string>}
     */
    async getVerdict(getCached) {
        if (getCached && this.verdict) return this.verdict;
        return this.verdict = await this.$client.getVerdict(this.aid, this.id);
    }

    /**
     * Returns the submission's entity tree.
     *
     * @param getCached if {@code true}, the the submission may return its cached values
     *                  instead of requesting new data from the server
     *
     * @returns {Promise<Submission.entity>}
     */
    async getEntity(getCached) {
        if (getCached && this.entity) return this.entity;
        const sub = await this.$client.getSubmission(this.aid, this.id);
        this.entity = sub.entity;
        return this.entity;
    }
}
