import fs from 'fs';
import Token from '../../Token';

/**
 * A token provider that reads the token from disk.
 */
export default class PersistentTokenAuthProvider {
    /**
     * Creates a new persistent token auth provider.
     *
     * @param path the path to the file where the token is stored
     */
    constructor(path) {
        this.path = path;
    }

    /**
     * Returns the token for the current user.
     *
     * @returns {Promise<Token>}
     */
    async getToken() {
        return new Token(await fs.promises.readFile(this.path, 'utf8'));
    }

    /**
     * Handles the server rejecting the provider's token.
     *
     * @returns {Promise<void>}
     */
    async handleInvalid() {
        throw new Error('Token is expired or invalid');
    }

    /**
     * Writes the token to disk.
     *
     * @param token the token
     *
     * @returns {Promise<void>}
     */
    async setToken(token) {
        await fs.promises.writeFile(this.path, token, {encoding: 'utf8'});
    }
}
