'use strict';

import Client from '../../Client';
import PersistentTokenAuthProvider from './PersistentTokenAuthProvider';
import NodeHttp from './NodeHttp';

/**
 * The library's entry point.
 */
export default class AuTA {
    /**
     * Creates a new Client instance.
     *
     * @param config the client configuration
     *
     * @returns {Client} the client
     */
    static client(config) {
        const authProvider = config.authProvider || new PersistentTokenAuthProvider(config.tokenFilePath);
        const nodeHttp = config.http || new NodeHttp(config.baseUrl, authProvider);
        return new Client(nodeHttp);
    }

    /**
     * Returns the version auta.js was built as.
     *
     * @returns {string} version
     */
    static version() {
        return VERSION;
    }
}

export {default as PersistentTokenAuthProvider} from './PersistentTokenAuthProvider';
export {default as NodeHttp} from './NodeHttp';
export {default as Severity} from '../../Severity';
export {default as EntityLevel} from '../../EntityLevel';
