'use strict';

import Client from '../../Client';
import BrowserLocallyCachedAuthProvider from './BrowserLocallyCachedAuthProvider';
import AjaxHttp from './AjaxHttp';

/**
 * The library's entry point.
 */
export default class AuTA {
    /**
     * Creates a new Client instance.
     *
     * @param config the client configuration
     *
     * @returns {Client} the client
     */
    static client(config) {
        const baseUrl = config.baseUrl || window.location.origin;
        const authProvider = config.authProvider || new BrowserLocallyCachedAuthProvider();
        const webHttp = config.http || new AjaxHttp(baseUrl, authProvider, config.httpDebug);
        return new Client(webHttp);
    }

    /**
     * Returns the version auta.js was built as.
     *
     * @returns {string} version
     */
    static version() {
        return VERSION;
    }

    static BrowserLocallyCachedAuthProvider() { return new BrowserLocallyCachedAuthProvider(); }
    static AjaxHttp(baseUrl, authProvider) { return new AjaxHttp(baseUrl, authProvider); }
}

export {default as BrowserLocallyCachedAuthProvider} from './BrowserLocallyCachedAuthProvider';
export {default as AjaxHttp} from './AjaxHttp';
export {default as Severity} from '../../Severity';
export {default as EntityLevel} from '../../EntityLevel';
