import Page from '../src/Page';
import Assignment from '../src/Assignment';

test('getSubmissions forwards to client', async () => {
    const submissionsPage = new Page([], 1, 20, 1);
    const aid = 'test assignment id';
    const page = 1;
    const pageSize = 20;

    const client = {
        async getSubmissions(aid, page, pageSize) {
            expect(aid).toEqual(aid);
            return submissionsPage;
        }
    };

    const assignment = new Assignment(aid, 'test assignment name', {}, client);
    const res = await assignment.getSubmissions(page, pageSize);
    expect(res).toBe(submissionsPage);
});

test('update forwards to client', async () => {
    const assignment = new Assignment('id!', 'test assignment name 2', {}, undefined);

    const client = {
        async updateAssignment(a) {
            expect(a).toBe(assignment);
        }
    };

    assignment.$client = client;

    await assignment.update();
});

test('create forwards to client', async () => {
    const ID = 'ID!!!';
    const assignment = new Assignment(undefined, 'test assignment name 2', {}, undefined);

    const client = {
        async addAssignment(a) {
            expect(a).toBe(assignment);
            return ID;
        }
    };

    assignment.$client = client;

    await assignment.create();
    expect(assignment.id).toEqual(ID);
});

test('delete forwards to the client', async () => {
    const ID = 'id 2';

    expect.assertions(1);

    const client = {
        async deleteAssignment(aid) {
            expect(aid).toEqual(ID);
            return { data: { } };
        }
    };

    const assignment = new Assignment(ID, 'name', { options: true }, client);
    await assignment.delete();


});
