import Client from '../src/Client';
import Token from '../src/Token';
import Assignment from "../src/Assignment";
import Course from "../src/Course"
import Page from "../src/Page";
import User from "../src/User";
import Metric from "../src/Metric";

const COURSE_RESPONSE = {
    "courses":[{
        "id": "cid",
        "name": "CT",
        "courseCode": "TI3306",
        "year": 2019,
        "instructorSet": [],
        "taSet": [
            "ta"
        ],
        "assignmentIds": [
            "aid1"
        ]
    }],
    "errors":[]
};

const ASSIGNMENT_LIST_RESPONSE = {
    "assignments": [
        {
            "id": "5b9e78852a75ef3bfa07925c",
            "name": "Assignment 3",
            "options": {
                "static": [
                    {
                        "name": "recursion",
                        "script": "(() => recursiveCalls => {\n    if (recursiveCalls.indexOf('factorial') === -1) {\n        fail('Factorial is not (direct) recursive');\n    }\n})()\n",
                        "formatter": "",
                        "options": {}
                    }
                ],
                "language": "x86_64",
                "dynamic": [
                    {
                        "name": "fragment count",
                        "script": "(() => counts => {\n    if (counts['fragment count'] === 0) {\n        fail('No analyzable files found. Make sure all source files have correct extensions and, if you have submitted an archive, you have uploaded a proper .zip file.');\n    }\n})()\n",
                        "formatter": "",
                        "options": {}
                    }
                ],
                "assignment name": "Assignment 3"
            },
            "priority": 0.0,
            "templateId": null,
            "deleted": false,
            "workUnitName": null,
            "workUnitId": null,
            "projectName": null
        },
        {
            "id": "5bb0c1d62a75ef087ee66c52",
            "name": "Stoffig",
            "options": {
                "static": [
                    {
                        "name": "method length",
                        "script": "(() => results => {\n    const warnThreshold = 20;\n    const max = 40;\n\n    for (let key in results) {\n        if (results[key] > max) {\n            fail(\n                key + ' has too many lines:'\n                + results[key] + ' > ' + max\n            );\n        } else if (results[key] > warnThreshold) {\n            warn(\n                key + ' has many lines:'\n                + results[key] + ' > ' + warnThreshold\n            );\n        }\n    }\n})();",
                        "formatter": "",
                        "options": {}
                    },
                    {
                        "name": "line length",
                        "script": "(() => results => {\n    const warnThreshold = 120;\n    const max = 150;\n\n    for (let i = 0; i < results.length; ++i) {\n        let lineno = i + 1;\n\n        if (results[i] > max) {\n            fail(\n                'Line ' + lineno + ' is too long: '\n                + results[i] + ' > ' + max\n            );\n        } else if (results[i] > warnThreshold) {\n            warn(\n                'Line ' + lineno + ' is very long: '\n                + results[i] + ' > ' + warnThreshold\n            );\n        }\n    }\n})();\n",
                        "formatter": "",
                        "options": {}
                    }
                ],
                "language": "python",
                "dynamic": [],
                "assignment name": "Stoffig"
            },
            "priority": 0.0,
            "templateId": null,
            "deleted": false,
            "workUnitName": null,
            "workUnitId": null,
            "projectName": null
        }
    ],
    "errors": []
};

const SUBMISSION_LIST_RESPONSE = {
    "numPages": 3,
    "submissions": [
        {
            "name": "/0fb6c31a0ada5f18409ae0674b6d5c50.log",
            "id": "5ba56b122a75ef2e3c279497",
            "identity": null
        },
        {
            "name": "/0fb6c31a0ada5f18409ae0674b6d5c50.log",
            "id": "5ba56b3e2a75ef2e3c279498",
            "identity": null
        },
        {
            "name": "/factorial uno.s",
            "id": "5ba56b6c2a75ef2e3c279499",
            "identity": null
        },
        {
            "name": "/0fb6c31a0ada5f18409ae0674b6d5c50.log",
            "id": "5ba56ba32a75ef2e3c27949a",
            "identity": null
        },
        {
            "name": "/0fb6c31a0ada5f18409ae0674b6d5c50.log",
            "id": "5ba56bc52a75ef2e3c27949b",
            "identity": null
        },
        {
            "name": "/0fb6c31a0ada5f18409ae0674b6d5c50.log",
            "id": "5ba56bed2a75ef2e3c27949c",
            "identity": null
        },
        {
            "name": "/0fb6c31a0ada5f18409ae0674b6d5c50.log",
            "id": "5ba56bfc2a75ef2e3c27949d",
            "identity": null
        },
        {
            "name": "/0fb6c31a0ada5f18409ae0674b6d5c50.log",
            "id": "5ba56c202a75ef2e3c27949e",
            "identity": null
        },
        {
            "name": "/factorial1.s",
            "id": "5ba610102a75ef1531ebe6f6",
            "identity": {
                "identifier" : "WOOP",
                "submissionId": "5ba610102a75ef1531ebe6f6",
            }
        }
    ],
    "page": 3,
    "errors": []
};

const ASSIGNMENT_RESPONSE = {
    "name": "Assignment 3",
    "options": {
        "static": [
            {
                "name": "recursion",
                "script": "(() => recursiveCalls => {\n    if (recursiveCalls.indexOf('factorial') === -1) {\n        fail('Factorial is not (direct) recursive');\n    }\n})()\n",
                "formatter": "",
                "options": {}
            }
        ],
        "language": "x86_64",
        "dynamic": [
            {
                "name": "fragment count",
                "script": "(() => counts => {\n    if (counts['fragment count'] === 0) {\n        fail('No analyzable files found. Make sure all source files have correct extensions and, if you have submitted an archive, you have uploaded a proper .zip file.');\n    }\n})()\n",
                "formatter": "",
                "options": {}
            }
        ],
        "assignment name": "Assignment 3"
    },
    "id": "5b9e78852a75ef3bfa07925c",
    "errors": []
};

const SUBMISSION_RESPONSE = {"identity":{"identifier":"hi", "submissionId":"5ce571b0edcd016535c3e09c"},"name":"/assignment5.zip","id":"5ce571b0edcd016535c3e09c","aid":"5ce571abedcd016535c3e09b","results":{"parent":null,"children":[{"parent":null,"children":[{"parent":null,"children":[],"name":"Boiler::getEnergyEfficiency( int rooms)","level":"METHOD","metrics":[{"value":23,"name":"METHOD_LENGTH"},{"value":6,"name":"CYCLOMATIC_COMPLEXITY"}],"fullName":"Boiler::getEnergyEfficiency( int rooms)","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":6.0}],"testEntity":false}],"name":"Boiler.java","level":"FILE","metrics":[],"fullName":"Boiler.java","reports":[],"testEntity":false},{"parent":null,"children":[{"parent":null,"children":[],"name":"HouseCatalog::HouseCatalog()","level":"METHOD","metrics":[{"value":1,"name":"CYCLOMATIC_COMPLEXITY"},{"value":4,"name":"METHOD_LENGTH"}],"fullName":"HouseCatalog::HouseCatalog()","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":1.0}],"testEntity":false},{"parent":null,"children":[],"name":"HouseCatalog::read( String fileName)","level":"METHOD","metrics":[{"value":2,"name":"CYCLOMATIC_COMPLEXITY"},{"value":18,"name":"METHOD_LENGTH"}],"fullName":"HouseCatalog::read( String fileName)","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":2.0}],"testEntity":false},{"parent":null,"children":[],"name":"HouseCatalog::housesCostAtMost( int price)","level":"METHOD","metrics":[{"value":3,"name":"CYCLOMATIC_COMPLEXITY"},{"value":12,"name":"METHOD_LENGTH"}],"fullName":"HouseCatalog::housesCostAtMost( int price)","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":3.0}],"testEntity":false},{"parent":null,"children":[],"name":"HouseCatalog::getHouses()","level":"METHOD","metrics":[{"value":1,"name":"CYCLOMATIC_COMPLEXITY"},{"value":4,"name":"METHOD_LENGTH"}],"fullName":"HouseCatalog::getHouses()","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":1.0}],"testEntity":false},{"parent":null,"children":[],"name":"HouseCatalog::toString()","level":"METHOD","metrics":[{"value":6,"name":"METHOD_LENGTH"},{"value":1,"name":"CYCLOMATIC_COMPLEXITY"}],"fullName":"HouseCatalog::toString()","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":1.0}],"testEntity":false},{"parent":null,"children":[],"name":"HouseCatalog::addHouse( House house)","level":"METHOD","metrics":[{"value":2,"name":"CYCLOMATIC_COMPLEXITY"},{"value":7,"name":"METHOD_LENGTH"}],"fullName":"HouseCatalog::addHouse( House house)","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":2.0}],"testEntity":false}],"name":"HouseCatalog.java","level":"FILE","metrics":[],"fullName":"HouseCatalog.java","reports":[],"testEntity":false},{"parent":null,"children":[{"parent":null,"children":[],"name":"Address::read( Scanner sc)","level":"METHOD","metrics":[{"value":1,"name":"CYCLOMATIC_COMPLEXITY"},{"value":8,"name":"METHOD_LENGTH"}],"fullName":"Address::read( Scanner sc)","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":1.0}],"testEntity":false},{"parent":null,"children":[],"name":"Address::equals( Object other)","level":"METHOD","metrics":[{"value":6,"name":"CYCLOMATIC_COMPLEXITY"},{"value":13,"name":"METHOD_LENGTH"}],"fullName":"Address::equals( Object other)","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":6.0}],"testEntity":false},{"parent":null,"children":[],"name":"Address::getNumber()","level":"METHOD","metrics":[{"value":1,"name":"CYCLOMATIC_COMPLEXITY"},{"value":4,"name":"METHOD_LENGTH"}],"fullName":"Address::getNumber()","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":1.0}],"testEntity":false},{"parent":null,"children":[],"name":"Address::getzipCode()","level":"METHOD","metrics":[{"value":1,"name":"CYCLOMATIC_COMPLEXITY"},{"value":4,"name":"METHOD_LENGTH"}],"fullName":"Address::getzipCode()","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":1.0}],"testEntity":false},{"parent":null,"children":[],"name":"Address::getStreet()","level":"METHOD","metrics":[{"value":1,"name":"CYCLOMATIC_COMPLEXITY"},{"value":4,"name":"METHOD_LENGTH"}],"fullName":"Address::getStreet()","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":1.0}],"testEntity":false},{"parent":null,"children":[],"name":"Address::getCity()","level":"METHOD","metrics":[{"value":1,"name":"CYCLOMATIC_COMPLEXITY"},{"value":4,"name":"METHOD_LENGTH"}],"fullName":"Address::getCity()","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":1.0}],"testEntity":false},{"parent":null,"children":[],"name":"Address::toString()","level":"METHOD","metrics":[{"value":5,"name":"METHOD_LENGTH"},{"value":1,"name":"CYCLOMATIC_COMPLEXITY"}],"fullName":"Address::toString()","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":1.0}],"testEntity":false},{"parent":null,"children":[],"name":"Address::Address( String street , int number , String zipCode , String city)","level":"METHOD","metrics":[{"value":1,"name":"CYCLOMATIC_COMPLEXITY"},{"value":7,"name":"METHOD_LENGTH"}],"fullName":"Address::Address( String street , int number , String zipCode , String city)","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":1.0}],"testEntity":false}],"name":"Address.java","level":"FILE","metrics":[],"fullName":"Address.java","reports":[],"testEntity":false},{"parent":null,"children":[{"parent":null,"children":[],"name":"RentalHouse::getAvailibilityText()","level":"METHOD","metrics":[{"value":2,"name":"CYCLOMATIC_COMPLEXITY"},{"value":12,"name":"METHOD_LENGTH"}],"fullName":"RentalHouse::getAvailibilityText()","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":2.0}],"testEntity":false},{"parent":null,"children":[],"name":"RentalHouse::getEnergyLabel( int nRooms , HeatingSystem system)","level":"METHOD","metrics":[{"value":18,"name":"CYCLOMATIC_COMPLEXITY"},{"value":48,"name":"METHOD_LENGTH"}],"fullName":"RentalHouse::getEnergyLabel( int nRooms , HeatingSystem system)","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":18.0}],"testEntity":false},{"parent":null,"children":[],"name":"RentalHouse::RentalHouse( Address address , int nRooms , int salePrice , boolean available , HeatingSystem heatingSystem)","level":"METHOD","metrics":[{"value":1,"name":"CYCLOMATIC_COMPLEXITY"},{"value":4,"name":"METHOD_LENGTH"}],"fullName":"RentalHouse::RentalHouse( Address address , int nRooms , int salePrice , boolean available , HeatingSystem heatingSystem)","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":1.0}],"testEntity":false}],"name":"RentalHouse.java","level":"FILE","metrics":[],"fullName":"RentalHouse.java","reports":[],"testEntity":false},{"parent":null,"children":[{"parent":null,"children":[],"name":"CentralHeating::getEnergyEfficiency( int rooms)","level":"METHOD","metrics":[{"value":24,"name":"METHOD_LENGTH"},{"value":11,"name":"CYCLOMATIC_COMPLEXITY"}],"fullName":"CentralHeating::getEnergyEfficiency( int rooms)","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":11.0}],"testEntity":false}],"name":"CentralHeating.java","level":"FILE","metrics":[],"fullName":"CentralHeating.java","reports":[],"testEntity":false},{"parent":null,"children":[],"name":"HeatingSystem.java","level":"FILE","metrics":[],"fullName":"HeatingSystem.java","reports":[],"testEntity":false},{"parent":null,"children":[{"parent":null,"children":[],"name":"House::toString()","level":"METHOD","metrics":[{"value":5,"name":"METHOD_LENGTH"},{"value":1,"name":"CYCLOMATIC_COMPLEXITY"}],"fullName":"House::toString()","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":1.0}],"testEntity":false},{"parent":null,"children":[],"name":"House::costsAtMost( int price)","level":"METHOD","metrics":[{"value":2,"name":"CYCLOMATIC_COMPLEXITY"},{"value":9,"name":"METHOD_LENGTH"}],"fullName":"House::costsAtMost( int price)","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":2.0}],"testEntity":false},{"parent":null,"children":[],"name":"House::House( Address address , int nRooms , int salePrice , boolean available , HeatingSystem heatingSystem)","level":"METHOD","metrics":[{"value":1,"name":"CYCLOMATIC_COMPLEXITY"},{"value":8,"name":"METHOD_LENGTH"}],"fullName":"House::House( Address address , int nRooms , int salePrice , boolean available , HeatingSystem heatingSystem)","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":1.0}],"testEntity":false},{"parent":null,"children":[],"name":"House::getNRooms()","level":"METHOD","metrics":[{"value":1,"name":"CYCLOMATIC_COMPLEXITY"},{"value":4,"name":"METHOD_LENGTH"}],"fullName":"House::getNRooms()","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":1.0}],"testEntity":false},{"parent":null,"children":[],"name":"House::getAddress()","level":"METHOD","metrics":[{"value":1,"name":"CYCLOMATIC_COMPLEXITY"},{"value":4,"name":"METHOD_LENGTH"}],"fullName":"House::getAddress()","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":1.0}],"testEntity":false},{"parent":null,"children":[],"name":"House::read( Scanner sc)","level":"METHOD","metrics":[{"value":24,"name":"METHOD_LENGTH"},{"value":7,"name":"CYCLOMATIC_COMPLEXITY"}],"fullName":"House::read( Scanner sc)","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":7.0}],"testEntity":false},{"parent":null,"children":[],"name":"House::getHeating( String heating)","level":"METHOD","metrics":[{"value":2,"name":"CYCLOMATIC_COMPLEXITY"},{"value":13,"name":"METHOD_LENGTH"}],"fullName":"House::getHeating( String heating)","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":2.0}],"testEntity":false},{"parent":null,"children":[],"name":"House::equals( Object other)","level":"METHOD","metrics":[{"value":5,"name":"CYCLOMATIC_COMPLEXITY"},{"value":13,"name":"METHOD_LENGTH"}],"fullName":"House::equals( Object other)","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":5.0}],"testEntity":false},{"parent":null,"children":[],"name":"House::getSalePrice()","level":"METHOD","metrics":[{"value":1,"name":"CYCLOMATIC_COMPLEXITY"},{"value":4,"name":"METHOD_LENGTH"}],"fullName":"House::getSalePrice()","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":1.0}],"testEntity":false},{"parent":null,"children":[],"name":"House::getAvailable()","level":"METHOD","metrics":[{"value":1,"name":"CYCLOMATIC_COMPLEXITY"},{"value":4,"name":"METHOD_LENGTH"}],"fullName":"House::getAvailable()","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":1.0}],"testEntity":false}],"name":"House.java","level":"FILE","metrics":[],"fullName":"House.java","reports":[],"testEntity":false},{"parent":null,"children":[{"parent":null,"children":[],"name":"HousingApplication::giveHouseNumber()","level":"METHOD","metrics":[{"value":1,"name":"CYCLOMATIC_COMPLEXITY"},{"value":7,"name":"METHOD_LENGTH"}],"fullName":"HousingApplication::giveHouseNumber()","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":1.0}],"testEntity":false},{"parent":null,"children":[],"name":"HousingApplication::giveHeatingsystem()","level":"METHOD","metrics":[{"value":18,"name":"METHOD_LENGTH"},{"value":5,"name":"CYCLOMATIC_COMPLEXITY"}],"fullName":"HousingApplication::giveHeatingsystem()","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":5.0}],"testEntity":false},{"parent":null,"children":[],"name":"HousingApplication::giveCity()","level":"METHOD","metrics":[{"value":6,"name":"METHOD_LENGTH"},{"value":1,"name":"CYCLOMATIC_COMPLEXITY"}],"fullName":"HousingApplication::giveCity()","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":1.0}],"testEntity":false},{"parent":null,"children":[],"name":"HousingApplication::main( String [ ] args)","level":"METHOD","metrics":[{"value":5,"name":"METHOD_LENGTH"},{"value":1,"name":"CYCLOMATIC_COMPLEXITY"}],"fullName":"HousingApplication::main( String [ ] args)","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":1.0}],"testEntity":false},{"parent":null,"children":[],"name":"HousingApplication::giveNRooms()","level":"METHOD","metrics":[{"value":6,"name":"METHOD_LENGTH"},{"value":1,"name":"CYCLOMATIC_COMPLEXITY"}],"fullName":"HousingApplication::giveNRooms()","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":1.0}],"testEntity":false},{"parent":null,"children":[],"name":"HousingApplication::givePrice()","level":"METHOD","metrics":[{"value":6,"name":"METHOD_LENGTH"},{"value":1,"name":"CYCLOMATIC_COMPLEXITY"}],"fullName":"HousingApplication::givePrice()","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":1.0}],"testEntity":false},{"parent":null,"children":[],"name":"HousingApplication::loop()","level":"METHOD","metrics":[{"value":47,"name":"CYCLOMATIC_COMPLEXITY"},{"value":200,"name":"METHOD_LENGTH"}],"fullName":"HousingApplication::loop()","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":47.0}],"testEntity":false},{"parent":null,"children":[],"name":"HousingApplication::giveStreet()","level":"METHOD","metrics":[{"value":6,"name":"METHOD_LENGTH"},{"value":1,"name":"CYCLOMATIC_COMPLEXITY"}],"fullName":"HousingApplication::giveStreet()","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":1.0}],"testEntity":false},{"parent":null,"children":[],"name":"HousingApplication::givePostalCode()","level":"METHOD","metrics":[{"value":6,"name":"METHOD_LENGTH"},{"value":1,"name":"CYCLOMATIC_COMPLEXITY"}],"fullName":"HousingApplication::givePostalCode()","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":1.0}],"testEntity":false},{"parent":null,"children":[],"name":"HousingApplication::WriteToFile( HouseCatalog housecatalog , String amount , String text)","level":"METHOD","metrics":[{"value":2,"name":"CYCLOMATIC_COMPLEXITY"},{"value":33,"name":"METHOD_LENGTH"}],"fullName":"HousingApplication::WriteToFile( HouseCatalog housecatalog , String amount , String text)","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":2.0}],"testEntity":false},{"parent":null,"children":[],"name":"HousingApplication::readfile()","level":"METHOD","metrics":[{"value":2,"name":"CYCLOMATIC_COMPLEXITY"},{"value":15,"name":"METHOD_LENGTH"}],"fullName":"HousingApplication::readfile()","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":2.0}],"testEntity":false}],"name":"HousingApplication.java","level":"FILE","metrics":[],"fullName":"HousingApplication.java","reports":[],"testEntity":false},{"parent":null,"children":[{"parent":null,"children":[],"name":"OwnerOccupiedHouse::OwnerOccupiedHouse( Address address , int nRooms , int salePrice , boolean available , HeatingSystem heatingSystem)","level":"METHOD","metrics":[{"value":1,"name":"CYCLOMATIC_COMPLEXITY"},{"value":4,"name":"METHOD_LENGTH"}],"fullName":"OwnerOccupiedHouse::OwnerOccupiedHouse( Address address , int nRooms , int salePrice , boolean available , HeatingSystem heatingSystem)","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":1.0}],"testEntity":false},{"parent":null,"children":[],"name":"OwnerOccupiedHouse::getAvailibilityText()","level":"METHOD","metrics":[{"value":2,"name":"CYCLOMATIC_COMPLEXITY"},{"value":13,"name":"METHOD_LENGTH"}],"fullName":"OwnerOccupiedHouse::getAvailibilityText()","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":2.0}],"testEntity":false},{"parent":null,"children":[],"name":"OwnerOccupiedHouse::getEnergyLabel( int nRooms , HeatingSystem system)","level":"METHOD","metrics":[{"value":18,"name":"CYCLOMATIC_COMPLEXITY"},{"value":48,"name":"METHOD_LENGTH"}],"fullName":"OwnerOccupiedHouse::getEnergyLabel( int nRooms , HeatingSystem system)","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":18.0}],"testEntity":false}],"name":"OwnerOccupiedHouse.java","level":"FILE","metrics":[],"fullName":"OwnerOccupiedHouse.java","reports":[],"testEntity":false}],"name":"root","level":"PROJECT","metrics":[],"fullName":"root","reports":[],"testEntity":false},"errors":[]};
const SUBMISSION_RESPONSE2 = {"identity":null,"name":"/assignment5.zip","id":"5ce571b0edcd016535c3e09c","aid":"5ce571abedcd016535c3e09b","results":{"parent":null,"children":[{"parent":null,"children":[{"parent":null,"children":[],"name":"Boiler::getEnergyEfficiency( int rooms)","level":"METHOD","metrics":[{"value":23,"name":"METHOD_LENGTH"},{"value":6,"name":"CYCLOMATIC_COMPLEXITY"}],"fullName":"Boiler::getEnergyEfficiency( int rooms)","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":6.0}],"testEntity":false}],"name":"Boiler.java","level":"FILE","metrics":[],"fullName":"Boiler.java","reports":[],"testEntity":false},{"parent":null,"children":[{"parent":null,"children":[],"name":"HouseCatalog::HouseCatalog()","level":"METHOD","metrics":[{"value":1,"name":"CYCLOMATIC_COMPLEXITY"},{"value":4,"name":"METHOD_LENGTH"}],"fullName":"HouseCatalog::HouseCatalog()","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":1.0}],"testEntity":false},{"parent":null,"children":[],"name":"HouseCatalog::read( String fileName)","level":"METHOD","metrics":[{"value":2,"name":"CYCLOMATIC_COMPLEXITY"},{"value":18,"name":"METHOD_LENGTH"}],"fullName":"HouseCatalog::read( String fileName)","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":2.0}],"testEntity":false},{"parent":null,"children":[],"name":"HouseCatalog::housesCostAtMost( int price)","level":"METHOD","metrics":[{"value":3,"name":"CYCLOMATIC_COMPLEXITY"},{"value":12,"name":"METHOD_LENGTH"}],"fullName":"HouseCatalog::housesCostAtMost( int price)","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":3.0}],"testEntity":false},{"parent":null,"children":[],"name":"HouseCatalog::getHouses()","level":"METHOD","metrics":[{"value":1,"name":"CYCLOMATIC_COMPLEXITY"},{"value":4,"name":"METHOD_LENGTH"}],"fullName":"HouseCatalog::getHouses()","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":1.0}],"testEntity":false},{"parent":null,"children":[],"name":"HouseCatalog::toString()","level":"METHOD","metrics":[{"value":6,"name":"METHOD_LENGTH"},{"value":1,"name":"CYCLOMATIC_COMPLEXITY"}],"fullName":"HouseCatalog::toString()","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":1.0}],"testEntity":false},{"parent":null,"children":[],"name":"HouseCatalog::addHouse( House house)","level":"METHOD","metrics":[{"value":2,"name":"CYCLOMATIC_COMPLEXITY"},{"value":7,"name":"METHOD_LENGTH"}],"fullName":"HouseCatalog::addHouse( House house)","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":2.0}],"testEntity":false}],"name":"HouseCatalog.java","level":"FILE","metrics":[],"fullName":"HouseCatalog.java","reports":[],"testEntity":false},{"parent":null,"children":[{"parent":null,"children":[],"name":"Address::read( Scanner sc)","level":"METHOD","metrics":[{"value":1,"name":"CYCLOMATIC_COMPLEXITY"},{"value":8,"name":"METHOD_LENGTH"}],"fullName":"Address::read( Scanner sc)","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":1.0}],"testEntity":false},{"parent":null,"children":[],"name":"Address::equals( Object other)","level":"METHOD","metrics":[{"value":6,"name":"CYCLOMATIC_COMPLEXITY"},{"value":13,"name":"METHOD_LENGTH"}],"fullName":"Address::equals( Object other)","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":6.0}],"testEntity":false},{"parent":null,"children":[],"name":"Address::getNumber()","level":"METHOD","metrics":[{"value":1,"name":"CYCLOMATIC_COMPLEXITY"},{"value":4,"name":"METHOD_LENGTH"}],"fullName":"Address::getNumber()","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":1.0}],"testEntity":false},{"parent":null,"children":[],"name":"Address::getzipCode()","level":"METHOD","metrics":[{"value":1,"name":"CYCLOMATIC_COMPLEXITY"},{"value":4,"name":"METHOD_LENGTH"}],"fullName":"Address::getzipCode()","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":1.0}],"testEntity":false},{"parent":null,"children":[],"name":"Address::getStreet()","level":"METHOD","metrics":[{"value":1,"name":"CYCLOMATIC_COMPLEXITY"},{"value":4,"name":"METHOD_LENGTH"}],"fullName":"Address::getStreet()","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":1.0}],"testEntity":false},{"parent":null,"children":[],"name":"Address::getCity()","level":"METHOD","metrics":[{"value":1,"name":"CYCLOMATIC_COMPLEXITY"},{"value":4,"name":"METHOD_LENGTH"}],"fullName":"Address::getCity()","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":1.0}],"testEntity":false},{"parent":null,"children":[],"name":"Address::toString()","level":"METHOD","metrics":[{"value":5,"name":"METHOD_LENGTH"},{"value":1,"name":"CYCLOMATIC_COMPLEXITY"}],"fullName":"Address::toString()","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":1.0}],"testEntity":false},{"parent":null,"children":[],"name":"Address::Address( String street , int number , String zipCode , String city)","level":"METHOD","metrics":[{"value":1,"name":"CYCLOMATIC_COMPLEXITY"},{"value":7,"name":"METHOD_LENGTH"}],"fullName":"Address::Address( String street , int number , String zipCode , String city)","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":1.0}],"testEntity":false}],"name":"Address.java","level":"FILE","metrics":[],"fullName":"Address.java","reports":[],"testEntity":false},{"parent":null,"children":[{"parent":null,"children":[],"name":"RentalHouse::getAvailibilityText()","level":"METHOD","metrics":[{"value":2,"name":"CYCLOMATIC_COMPLEXITY"},{"value":12,"name":"METHOD_LENGTH"}],"fullName":"RentalHouse::getAvailibilityText()","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":2.0}],"testEntity":false},{"parent":null,"children":[],"name":"RentalHouse::getEnergyLabel( int nRooms , HeatingSystem system)","level":"METHOD","metrics":[{"value":18,"name":"CYCLOMATIC_COMPLEXITY"},{"value":48,"name":"METHOD_LENGTH"}],"fullName":"RentalHouse::getEnergyLabel( int nRooms , HeatingSystem system)","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":18.0}],"testEntity":false},{"parent":null,"children":[],"name":"RentalHouse::RentalHouse( Address address , int nRooms , int salePrice , boolean available , HeatingSystem heatingSystem)","level":"METHOD","metrics":[{"value":1,"name":"CYCLOMATIC_COMPLEXITY"},{"value":4,"name":"METHOD_LENGTH"}],"fullName":"RentalHouse::RentalHouse( Address address , int nRooms , int salePrice , boolean available , HeatingSystem heatingSystem)","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":1.0}],"testEntity":false}],"name":"RentalHouse.java","level":"FILE","metrics":[],"fullName":"RentalHouse.java","reports":[],"testEntity":false},{"parent":null,"children":[{"parent":null,"children":[],"name":"CentralHeating::getEnergyEfficiency( int rooms)","level":"METHOD","metrics":[{"value":24,"name":"METHOD_LENGTH"},{"value":11,"name":"CYCLOMATIC_COMPLEXITY"}],"fullName":"CentralHeating::getEnergyEfficiency( int rooms)","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":11.0}],"testEntity":false}],"name":"CentralHeating.java","level":"FILE","metrics":[],"fullName":"CentralHeating.java","reports":[],"testEntity":false},{"parent":null,"children":[],"name":"HeatingSystem.java","level":"FILE","metrics":[],"fullName":"HeatingSystem.java","reports":[],"testEntity":false},{"parent":null,"children":[{"parent":null,"children":[],"name":"House::toString()","level":"METHOD","metrics":[{"value":5,"name":"METHOD_LENGTH"},{"value":1,"name":"CYCLOMATIC_COMPLEXITY"}],"fullName":"House::toString()","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":1.0}],"testEntity":false},{"parent":null,"children":[],"name":"House::costsAtMost( int price)","level":"METHOD","metrics":[{"value":2,"name":"CYCLOMATIC_COMPLEXITY"},{"value":9,"name":"METHOD_LENGTH"}],"fullName":"House::costsAtMost( int price)","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":2.0}],"testEntity":false},{"parent":null,"children":[],"name":"House::House( Address address , int nRooms , int salePrice , boolean available , HeatingSystem heatingSystem)","level":"METHOD","metrics":[{"value":1,"name":"CYCLOMATIC_COMPLEXITY"},{"value":8,"name":"METHOD_LENGTH"}],"fullName":"House::House( Address address , int nRooms , int salePrice , boolean available , HeatingSystem heatingSystem)","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":1.0}],"testEntity":false},{"parent":null,"children":[],"name":"House::getNRooms()","level":"METHOD","metrics":[{"value":1,"name":"CYCLOMATIC_COMPLEXITY"},{"value":4,"name":"METHOD_LENGTH"}],"fullName":"House::getNRooms()","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":1.0}],"testEntity":false},{"parent":null,"children":[],"name":"House::getAddress()","level":"METHOD","metrics":[{"value":1,"name":"CYCLOMATIC_COMPLEXITY"},{"value":4,"name":"METHOD_LENGTH"}],"fullName":"House::getAddress()","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":1.0}],"testEntity":false},{"parent":null,"children":[],"name":"House::read( Scanner sc)","level":"METHOD","metrics":[{"value":24,"name":"METHOD_LENGTH"},{"value":7,"name":"CYCLOMATIC_COMPLEXITY"}],"fullName":"House::read( Scanner sc)","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":7.0}],"testEntity":false},{"parent":null,"children":[],"name":"House::getHeating( String heating)","level":"METHOD","metrics":[{"value":2,"name":"CYCLOMATIC_COMPLEXITY"},{"value":13,"name":"METHOD_LENGTH"}],"fullName":"House::getHeating( String heating)","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":2.0}],"testEntity":false},{"parent":null,"children":[],"name":"House::equals( Object other)","level":"METHOD","metrics":[{"value":5,"name":"CYCLOMATIC_COMPLEXITY"},{"value":13,"name":"METHOD_LENGTH"}],"fullName":"House::equals( Object other)","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":5.0}],"testEntity":false},{"parent":null,"children":[],"name":"House::getSalePrice()","level":"METHOD","metrics":[{"value":1,"name":"CYCLOMATIC_COMPLEXITY"},{"value":4,"name":"METHOD_LENGTH"}],"fullName":"House::getSalePrice()","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":1.0}],"testEntity":false},{"parent":null,"children":[],"name":"House::getAvailable()","level":"METHOD","metrics":[{"value":1,"name":"CYCLOMATIC_COMPLEXITY"},{"value":4,"name":"METHOD_LENGTH"}],"fullName":"House::getAvailable()","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":1.0}],"testEntity":false}],"name":"House.java","level":"FILE","metrics":[],"fullName":"House.java","reports":[],"testEntity":false},{"parent":null,"children":[{"parent":null,"children":[],"name":"HousingApplication::giveHouseNumber()","level":"METHOD","metrics":[{"value":1,"name":"CYCLOMATIC_COMPLEXITY"},{"value":7,"name":"METHOD_LENGTH"}],"fullName":"HousingApplication::giveHouseNumber()","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":1.0}],"testEntity":false},{"parent":null,"children":[],"name":"HousingApplication::giveHeatingsystem()","level":"METHOD","metrics":[{"value":18,"name":"METHOD_LENGTH"},{"value":5,"name":"CYCLOMATIC_COMPLEXITY"}],"fullName":"HousingApplication::giveHeatingsystem()","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":5.0}],"testEntity":false},{"parent":null,"children":[],"name":"HousingApplication::giveCity()","level":"METHOD","metrics":[{"value":6,"name":"METHOD_LENGTH"},{"value":1,"name":"CYCLOMATIC_COMPLEXITY"}],"fullName":"HousingApplication::giveCity()","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":1.0}],"testEntity":false},{"parent":null,"children":[],"name":"HousingApplication::main( String [ ] args)","level":"METHOD","metrics":[{"value":5,"name":"METHOD_LENGTH"},{"value":1,"name":"CYCLOMATIC_COMPLEXITY"}],"fullName":"HousingApplication::main( String [ ] args)","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":1.0}],"testEntity":false},{"parent":null,"children":[],"name":"HousingApplication::giveNRooms()","level":"METHOD","metrics":[{"value":6,"name":"METHOD_LENGTH"},{"value":1,"name":"CYCLOMATIC_COMPLEXITY"}],"fullName":"HousingApplication::giveNRooms()","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":1.0}],"testEntity":false},{"parent":null,"children":[],"name":"HousingApplication::givePrice()","level":"METHOD","metrics":[{"value":6,"name":"METHOD_LENGTH"},{"value":1,"name":"CYCLOMATIC_COMPLEXITY"}],"fullName":"HousingApplication::givePrice()","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":1.0}],"testEntity":false},{"parent":null,"children":[],"name":"HousingApplication::loop()","level":"METHOD","metrics":[{"value":47,"name":"CYCLOMATIC_COMPLEXITY"},{"value":200,"name":"METHOD_LENGTH"}],"fullName":"HousingApplication::loop()","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":47.0}],"testEntity":false},{"parent":null,"children":[],"name":"HousingApplication::giveStreet()","level":"METHOD","metrics":[{"value":6,"name":"METHOD_LENGTH"},{"value":1,"name":"CYCLOMATIC_COMPLEXITY"}],"fullName":"HousingApplication::giveStreet()","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":1.0}],"testEntity":false},{"parent":null,"children":[],"name":"HousingApplication::givePostalCode()","level":"METHOD","metrics":[{"value":6,"name":"METHOD_LENGTH"},{"value":1,"name":"CYCLOMATIC_COMPLEXITY"}],"fullName":"HousingApplication::givePostalCode()","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":1.0}],"testEntity":false},{"parent":null,"children":[],"name":"HousingApplication::WriteToFile( HouseCatalog housecatalog , String amount , String text)","level":"METHOD","metrics":[{"value":2,"name":"CYCLOMATIC_COMPLEXITY"},{"value":33,"name":"METHOD_LENGTH"}],"fullName":"HousingApplication::WriteToFile( HouseCatalog housecatalog , String amount , String text)","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":2.0}],"testEntity":false},{"parent":null,"children":[],"name":"HousingApplication::readfile()","level":"METHOD","metrics":[{"value":2,"name":"CYCLOMATIC_COMPLEXITY"},{"value":15,"name":"METHOD_LENGTH"}],"fullName":"HousingApplication::readfile()","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":2.0}],"testEntity":false}],"name":"HousingApplication.java","level":"FILE","metrics":[],"fullName":"HousingApplication.java","reports":[],"testEntity":false},{"parent":null,"children":[{"parent":null,"children":[],"name":"OwnerOccupiedHouse::OwnerOccupiedHouse( Address address , int nRooms , int salePrice , boolean available , HeatingSystem heatingSystem)","level":"METHOD","metrics":[{"value":1,"name":"CYCLOMATIC_COMPLEXITY"},{"value":4,"name":"METHOD_LENGTH"}],"fullName":"OwnerOccupiedHouse::OwnerOccupiedHouse( Address address , int nRooms , int salePrice , boolean available , HeatingSystem heatingSystem)","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":1.0}],"testEntity":false},{"parent":null,"children":[],"name":"OwnerOccupiedHouse::getAvailibilityText()","level":"METHOD","metrics":[{"value":2,"name":"CYCLOMATIC_COMPLEXITY"},{"value":13,"name":"METHOD_LENGTH"}],"fullName":"OwnerOccupiedHouse::getAvailibilityText()","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":2.0}],"testEntity":false},{"parent":null,"children":[],"name":"OwnerOccupiedHouse::getEnergyLabel( int nRooms , HeatingSystem system)","level":"METHOD","metrics":[{"value":18,"name":"CYCLOMATIC_COMPLEXITY"},{"value":48,"name":"METHOD_LENGTH"}],"fullName":"OwnerOccupiedHouse::getEnergyLabel( int nRooms , HeatingSystem system)","reports":[{"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":18.0}],"testEntity":false}],"name":"OwnerOccupiedHouse.java","level":"FILE","metrics":[],"fullName":"OwnerOccupiedHouse.java","reports":[],"testEntity":false}],"name":"root","level":"PROJECT","metrics":[],"fullName":"root","reports":[],"testEntity":false},"errors":[]};

const METRICS_RESPONSE = {
    metrics: [
        {
            "script_presets": [
                {
                    "name": "Warn if >7, fail if >10",
                    "script": "(() => results => {\n    const warnThreshold = 7;\n    const max = 10;\n\n    for (let key in results) {\n        if (results[key] > max) {\n            fail(\n                key + ' has too high cyclomatic complexity:'\n                + results[key] + ' > ' + max\n            );\n        } else if (results[key] > warnThreshold) {\n            warn(\n                key + ' has high cyclomatic complexity:'\n                + results[key] + ' > ' + warnThreshold\n            );\n        }\n    }\n})();"
                }
            ],
            "internal_name": "cyclomatic complexity",
            "languages": [
                "cpp",
                "scala",
                "java",
                "python",
                "c"
            ],
            "name": "Cyclomatic complexity",
            "class": "static"
        },
        {
            "script_presets": [
                {
                    "name": "Warn if >30, fail if >40",
                    "script": "(() => results => {\n    const warnThreshold = 30;\n    const max = 40;\n\n    for (let key in results) {\n        if (results[key] > max) {\n            fail(\n                key + ' has too many lines:'\n                + results[key] + ' > ' + max\n            );\n        } else if (results[key] > warnThreshold) {\n            warn(\n                key + ' has many lines:'\n                + results[key] + ' > ' + warnThreshold\n            );\n        }\n    }\n})();"
                }
            ],
            "internal_name": "method length",
            "languages": [
                "cpp",
                "scala",
                "java",
                "python",
                "c"
            ],
            "name": "Method length",
            "class": "static"
        },
        {
            "script_presets": [
                {
                    "name": "Fail if <3, warn if <5",
                    "script": "(() => results => {\n    const warnThreshold = 5;\n    const min = 3;\n\n    if (results < min) {\n         fail(\n             'Has too low comment ratio:'\n             + results + ' < ' + min\n        );\n     } else if (results < warnThreshold) {\n         warn(\n             'Has low comment ration:'\n             + results + ' < ' + warnThreshold\n        );\n     }\n})();\n"
                }
            ],
            "internal_name": "comment count",
            "languages": [
                "cpp",
                "c",
                "java"
            ],
            "name": "Comment count",
            "class": "static"
        },
        {
            "script_presets": [
                {
                    "name": "Warn if >350, fail if >500",
                    "script": "(() => results => {\n    const warnThreshold = 350;\n    const max = 500;\n\n    if (results > max) {\n         fail(\n             'Has too many lines of code:'\n             + results + ' > ' + max\n        );\n     } else if (results > warnThreshold) {\n         warn(\n             'Has many lines of code:'\n             + results + ' > ' + warnThreshold\n        );\n     }\n})();\n"
                }
            ],
            "internal_name": "lines of code",
            "languages": [
                "cpp",
                "c",
                "java"
            ],
            "name": "Lines of code",
            "class": "static"
        },
        {
            "script_presets": [
                {
                    "name": "Warn if incomplete",
                    "script": "(() => results => {\n    for (let key in results) {\n        if (results[key].length > 0) {            warn(\n               key + ' has the following JavaDoc violations: ' + results[key]            );\n        }    }\n})();"
                },
                {
                    "name": "Fail if incomplete",
                    "script": "(() => results => {\n    for (let key in results) {\n        if (results[key].length > 0) {            fail(\n               key + ' has the following JavaDoc violations: ' + results[key]            );\n        }    }\n})();"
                }
            ],
            "internal_name": "javadoc violations",
            "languages": [
                "java"
            ],
            "name": "Javadoc violations",
            "class": "static"
        }
    ],
    "errors": []
};

const WORKER_TOKEN_RESPONSE = {
    "token": "848A40C532924DE78BDEDFA670EDEB9608FDB4578794C41F1835BBFF60027180C7821BEC2574F1438AC2990DDCC6149801D31D572D827A7C6ABFBA775AF1A2A6",
    "errors": []
};

const SETTINGS_RESPONSE = {
    "settings": {
        "security check interval": 30,
        "bcrypt work factor": 13.0,
        "token management interval": 5.0,
        "mongodb port": 27017,
        "allowed API origins": [
            "http://localhost:8080",
            "http://localhost:8081"
        ],
        "_worker account name": "_xXx_JobSlay3r69_xXx_",
        "worker ping interval": 20,
        "version": "0.1.0",
        "worker ping timeout": 30.0,
        "token management interval unit": "MINUTES",
        "worker port": 12728.0,
        "mongodb host": "10.1.4.1",
        "build date": "1.557233141E9",
        "api port": "8080",
        "security check interval unit": "MINUTES",
        "file store path": "/srv/auta/uploads",
        "mongodb database": "results",
        "token length": 512.0,
        "authentication database path": "/srv/auta/auth.db",
        "cpm submission url": "https://cpm.ewi.tudelft.nl/projects/scriptResult.php"
    },
    "errors": []
};
test('getUsers', async () => {
    const page = 1;
    const pageSize = 25;
    const response = {"numPages":1,"hasPrevious":false,"hasNext":false,"page":1,"errors":[],"users":[{"password":"fake password hash","username":"student","authorities":[{"authority":"ROLE_TEACHER"}],"accountNonExpired":true,"accountNonLocked":true,"credentialsNonExpired":true,"enabled":true}]};
    expect.assertions(2);
    const http = {
        get: url => new Promise(resolve => {
            expect(url).toEqual(`/api/v1/users?page=${page}&size=${pageSize}`);
            resolve({data: response});
        })
    };

    const client = new Client(http);
    const returnedPage = await client.getUsers(page, pageSize);

    expect(returnedPage).toEqual(new Page([new User('student',[{"authority":"ROLE_TEACHER"}], true)], page, pageSize, 1));

});
test('registerUser', () => {
    expect.assertions(2);
    const username = "USER";
    const password = "hunter2";
    const authority= "ROLE_TA";
    const http = {
        post: (url, data) => new Promise(resolve => {
            expect(url).toEqual(`/api/v1/user/register`);
            expect(data).toEqual({username, password, authority})
            resolve({data: {errors: []}});
        })
    };

    const client = new Client(http);
    return client.registerUser(username, password, authority);
});

test('updateUser', () => {
    expect.assertions(2);
    const username = "USER";
    const authority = "ROLE_TA";
    const enabled = false;
    const http = {
        put: (url, data) => new Promise(resolve => {
            expect(url).toEqual(`/api/v1/user`);
            expect(data).toEqual({username, authority, enabled})
            resolve({data: {errors: []}});
        })
    };

    const client = new Client(http);
    return client.updateUser(username, authority, enabled);
});

test('updatePassword', () => {
    expect.assertions(2);
    const username = "USER";
    const password = "hunter2";
    const oldPassword = "hunter3";
    const http = {
        put: (url, data) => new Promise(resolve => {
            expect(url).toEqual(`/api/v1/user/password`);
            expect(data).toEqual({username, oldPassword, password})
            resolve({data: {errors: []}});
        })
    };

    const client = new Client(http);
    return client.updatePassword(username, oldPassword, password);
});

test('connect', () => {
    const http = {
        get: url => new Promise(resolve => {
            expect(url).toEqual('/api/v1/user');
            resolve({data: {name: 'test user', enabled: true, authorities: ['LOSER']}});
        })
    };

    const client = new Client(http);
    return client.connect().then(() => {
        expect(client.user.name).toEqual('test user');
    });
});

test('connect twice', () => {
    let invocations = 0;
    const http = {
        get: url => new Promise(resolve => {
            ++invocations;
            expect(url).toEqual('/api/v1/user');
            resolve({data: {name: 'test user', enabled: true, authorities: ['LOSER']}});
        })
    };

    const client = new Client(http);
    return client.connect().then(() => client.connect().then(() => {
        expect(client.user.name).toEqual('test user');
        expect(invocations).toBe(1);
    }));
});


test('getWorkers', () => {
    const http = {
        get: url => new Promise(resolve => {
            expect(url).toEqual('/api/v1/workers');
            resolve({data: {workers: [], errors: []}});
        })
    };

    const client = new Client(http);
    return client.getWorkers().then(workers => {
        expect(workers).toEqual([]);
    });
});

test('getCourses', () => {
    expect.assertions(2);
    const http = {
        get: url => new Promise(resolve => {
            expect(url).toEqual('/api/v1/course');
            resolve({data:COURSE_RESPONSE});
        })
    };

    const client = new Client(http);
    const course = new Course("cid", "CT", "TI3306", 2019, ["ta"], [], ["aid1"], client);
    return client.getCourses().then(courses => {
        expect(courses).toEqual([course]);
    });
});

test('removeUserFromCourse', () => {
    const ID = "CID";
    const USERNAME = "arnold";
    const ROLE = "ROLE_TA";
    expect.assertions(1);
    const http = {
        delete: url => new Promise(resolve => {
            expect(url).toEqual(`/api/v1/course/${ID}/user/${ROLE}/${USERNAME}`);
            resolve({});
        })
    };

    const client = new Client(http);
    const course = new Course(ID, "CT", "TI3306", 2019, ["ta"], [], ["aid1"], client);
    return client.removeUserFromCourse(course, USERNAME, ROLE);
});

test('removeAssignmentFromCourse', () => {
    const ID = "CID";
    const AID = "aid";
    expect.assertions(1);
    const http = {
        delete: url => new Promise(resolve => {
            expect(url).toEqual(`/api/v1/course/${ID}/assignment/${AID}`);
            resolve({});
        })
    };

    const client = new Client(http);
    const course = new Course(ID, "CT", "TI3306", 2019, ["ta"], [], ["aid1"], client);
    return client.removeAssignmentFromCourse(course, AID);
});

test('getCourseMissing', () => {
    expect.assertions(2);
    const http = {
        get: url => new Promise(resolve => {
            expect(url).toEqual('/api/v1/course');
            resolve({data:COURSE_RESPONSE});
        })
    };

    const client = new Client(http);
    return client.getCourse("missingId").then(course => {
        expect(course).toBeUndefined();
    });
});


test('getCourseNotMissing', () => {
    expect.assertions(2);
    const http = {
        get: url => new Promise(resolve => {
            expect(url).toEqual('/api/v1/course');
            resolve({data:COURSE_RESPONSE});
        })
    };

    const client = new Client(http);
    const course = new Course("cid", "CT", "TI3306", 2019, ["ta"], [], ["aid1"], client);
    return client.getCourse("cid").then(c => {
        expect(c).toEqual(course);
    });
});


test('updateCourse', () => {
    const ID = "cid";
    const course = new Course(ID, "CT", "TI3306", 2019, ["ta"], [], ["aid1"], undefined);

    expect.assertions(2);
    const http = {
        put: (url, req) => new Promise(resolve => {
            expect(url).toEqual(`/api/v1/course/${ID}`);
            expect(req).toEqual(course.serialize());
            resolve({});
        })
    };
    const client = new Client(http);
    course.$client=client;
    return client.updateCourse(course);
});

test('addUserToCourse', () => {
    const ID = "cid";
    const USERNAME = "bob";
    const ROLE = "ROLE_TA";

    expect.assertions(2);
    const http = {
        put: (url, req) => new Promise(resolve => {
            expect(url).toEqual(`/api/v1/course/${ID}/user`);
            expect(req).toEqual({role:ROLE, username:USERNAME});
            resolve({});
        })
    };

    const client = new Client(http);
    const course = new Course(ID,"CT", "TI3306", 2019, ["ta"], [], ["aid1"], client);
    return client.addUserToCourse(course, USERNAME, ROLE);
});


test('addAssignmentToCourse', () => {
    const ID = "cid";
    const AID = "aid";

    expect.assertions(2);
    const http = {
        put: (url, req) => new Promise(resolve => {
            expect(url).toEqual(`/api/v1/course/${ID}/assignment`);
            expect(req).toEqual({aid:AID});
            resolve({});
        })
    };

    const client = new Client(http);
    const course = new Course(ID,"CT", "TI3306", 2019, ["ta"], [], ["aid1"], client);
    return client.addAssignmentToCourse(course, AID);
});

test('deleteCourse', () => {
    const ID = "cid";

    expect.assertions(1);
    const http = {
        delete: url => new Promise(resolve => {
            expect(url).toEqual(`/api/v1/course/${ID}`);
            resolve({});
        })
    };

    const client = new Client(http);
    return client.deleteCourse(ID);
});

test('add new course', () => {
    const COURSE_NAME = 'test course';
    const COURSE_CODE = 'test course code';
    const COURSE_YEAR = 'test course year';
    const COURSE_ID = 'unique';

    expect.assertions(6);
    const http = {
        async post(url, body, unauth) {
            expect(url).toEqual('/api/v1/course');
            expect(body).not.toBeInstanceOf(Assignment);
            expect(unauth).toBeFalsy();

            return { data: { id: COURSE_ID } };
        }
    };

    const client = new Client(http);

    const course = client.createNewCourse(COURSE_NAME, COURSE_CODE, COURSE_YEAR);
    expect(course.id).toBeUndefined();
    return client.addCourse(course).then(id => {
        expect(course.id).toEqual(COURSE_ID);
        expect(id).toEqual(COURSE_ID);
    });
});

test('login', () => {
    const tok = new Token('tok', 'yesterday');
    const username = 'username';
    const password = 'password';

    const authProvider = {
        getToken: canFail => new Promise(resolve => {
            expect(canFail).toBe(true);
            resolve(new Token(undefined, undefined));
        }),

        setToken: token => new Promise(resolve => {
            expect(token).toEqual(tok);
            resolve();
        })
    };

    const http = {
        authProvider,
        post: (url, req, unauth) => new Promise(resolve => {
            expect(url).toEqual('/api/v1/user/login');
            expect(unauth).toBe(true);
            expect(req).toEqual({username, password});
            resolve({data: tok});
        }),
        delete: _ => { throw new Error('Operation not allowed during this test'); }
    };

    const client = new Client(http);
    return client.login(username, password).then(token => expect(token).toEqual(tok));
});

test('login, but first log out', () => {
    const tok = new Token('tok', 'yesterday');
    const username = 'username';
    const password = 'password';

    const authProvider = {
        getToken: canFail => new Promise(resolve => {
            expect(canFail).toBe(true);
            resolve(new Token('old', 'not yet'));
        }),

        setToken: token => new Promise(resolve => {
            resolve();
        })
    };

    let loggedin = true;

    const http = {
        authProvider,
        post: (url, req, unauth) => new Promise(resolve => {
            expect(url).toEqual('/api/v1/user/login');
            expect(unauth).toBe(true);
            expect(req).toEqual({username, password});
            resolve({data: tok});
        }),
        delete: (url, unauth) => new Promise(resolve => {
            expect(url).toEqual('/api/v1/user/logout');
            expect(!!unauth).toBe(false);
            loggedin = false;
            resolve({data: {}});
        })
    };

    const client = new Client(http);
    return client.login(username, password).then(token => {
        expect(token).toEqual(tok);
        expect(loggedin).toBe(false);
    });
});

test('get assignments', () => {
    const http = {
        get(url, unauth) {
            expect(url).toEqual('/api/v1/assignment');
            expect(unauth).toBeFalsy();

            return new Promise(resolve => resolve({data: ASSIGNMENT_LIST_RESPONSE}));
        }
    };

    const client = new Client(http);
    return client.getAssignments().then(assignments => {
        expect(assignments).toHaveLength(2);
        expect(assignments[0].id).toEqual(ASSIGNMENT_LIST_RESPONSE.assignments[0].id);
        expect(assignments[1].name).toEqual(ASSIGNMENT_LIST_RESPONSE.assignments[1].name);
    });
});

test('get submissions', () => {
    expect.assertions(9);

    const aid = 'test_id';
    const page = 3;
    const pageSize = 10;

    const http = {
        get(url, unauth) {
            expect(url).toEqual(`/api/v1/assignment/${aid}/submission?page=${page}&size=${pageSize}`);
            expect(unauth).toBeFalsy();

            return new Promise(resolve => resolve({data: SUBMISSION_LIST_RESPONSE}));
        }
    };

    const client = new Client(http);
    return client.getSubmissions(aid, page, pageSize).then(submissionPage => {
        expect(submissionPage.page).toBe(page);
        expect(submissionPage.numPages).toBe(3);
        expect(submissionPage.pageSize).toBe(pageSize);
        expect(submissionPage.data[0].identity).toEqual(SUBMISSION_LIST_RESPONSE.submissions[0].identity);
        expect(submissionPage.data[8].identity).toEqual(SUBMISSION_LIST_RESPONSE.submissions[8].identity.identifier);
        expect(submissionPage.data[0].id).toEqual(SUBMISSION_LIST_RESPONSE.submissions[0].id);
        expect(submissionPage.data[4].name).toEqual(SUBMISSION_LIST_RESPONSE.submissions[4].name);
    });
});

test('get verdict', () => {
    const aid = 'test aa';
    const sid = 'test ss';

    const http = {
        async get(url, unauth) {
            expect(url).toEqual(`/api/v1/assignment/${aid}/submission/${sid}/verdict`);
            expect(unauth).toBeFalsy();

            return {data: {"verdict":"pass","errors":[]}};
        }
    };

    const client = new Client(http);
    return client.getVerdict(aid, sid).then(verdict => expect(verdict).toEqual('pass'));
});

test('get assignment', () => {
    const aid = ASSIGNMENT_RESPONSE.id;

    const http = {
        async get(url, unauth) {
            expect(url).toEqual(`/api/v1/assignment/${aid}`);
            expect(unauth).toBeFalsy();
            return {data: ASSIGNMENT_RESPONSE};
        }
    };

    const client = new Client(http);
    return client.getAssignment(aid).then(assignment => {
        expect(assignment.id).toEqual(ASSIGNMENT_RESPONSE.id);
        expect(assignment.name).toEqual(ASSIGNMENT_RESPONSE.name);
        expect(assignment.options).toEqual(ASSIGNMENT_RESPONSE.options);
    });
});

test('get submission', () => {
    expect.assertions(6);
    const aid = SUBMISSION_RESPONSE.aid;
    const sid = SUBMISSION_RESPONSE.id;

    const http = {
        async get(url, unauth) {
            expect(url).toEqual(`/api/v1/assignment/${aid}/submission/${sid}`);
            expect(unauth).toBeFalsy();
            return {data: SUBMISSION_RESPONSE};
        }
    };

    const client = new Client(http);
    return client.getSubmission(aid, sid).then(submission => {
        expect(submission.id).toEqual(SUBMISSION_RESPONSE.id);
        expect(submission.identity).toEqual(SUBMISSION_RESPONSE.identity.identifier);
        expect(submission.name).toEqual(SUBMISSION_RESPONSE.name);
        expect(submission.report).toEqual(SUBMISSION_RESPONSE.report);
    });
});

test('get submission null identifier', () => {
    expect.assertions(6);
    const aid = SUBMISSION_RESPONSE2.aid;
    const sid = SUBMISSION_RESPONSE2.id;

    const http = {
        async get(url, unauth) {
            expect(url).toEqual(`/api/v1/assignment/${aid}/submission/${sid}`);
            expect(unauth).toBeFalsy();
            return {data: SUBMISSION_RESPONSE2};
        }
    };

    const client = new Client(http);
    return client.getSubmission(aid, sid).then(submission => {
        expect(submission.id).toEqual(SUBMISSION_RESPONSE2.id);
        expect(submission.identity).toEqual(SUBMISSION_RESPONSE2.identity);
        expect(submission.name).toEqual(SUBMISSION_RESPONSE2.name);
        expect(submission.report).toEqual(SUBMISSION_RESPONSE2.report);
    });
});



test('get metrics', () => {
    const http = {
        async get(url, unauth) {
            expect(url).toEqual('/api/v1/worker/metrics');
            expect(unauth).toBeFalsy();

            return {data: METRICS_RESPONSE};
        }
    };

    const client = new Client(http);
    return client.getMetrics().then(metrics => {
        expect(metrics[0].name).toEqual('Cyclomatic complexity');
    });
});

test('get token', () => {
    const http = {
        async get(url, unauth) {
            expect(url).toEqual('/api/v1/worker/token');
            expect(unauth).toBeFalsy();

            return {data: WORKER_TOKEN_RESPONSE};
        }
    };

    const client = new Client(http);
    return client.getWorkerToken().then(token => {
        expect(token).toEqual(WORKER_TOKEN_RESPONSE.token);
    });
});

test('add new assignment', () => {
    const ASSIGNMENT_NAME = 'test assignment';
    const ASSIGNMENT_ID = 'test assignment unique ID!!!';

    const http = {
        async post(url, body, unauth) {
            expect(url).toEqual('/api/v1/assignment');
            expect(body).not.toBeInstanceOf(Assignment);
            expect(unauth).toBeFalsy();

            return { data: { id: ASSIGNMENT_ID } };
        }
    };

    const client = new Client(http);

    const assignment = client.createNewAssignment(ASSIGNMENT_NAME, { prop: 'value' });
    expect(assignment.id).toBeUndefined();
    return client.addAssignment(assignment).then(id => {
        expect(assignment.id).toEqual(ASSIGNMENT_ID);
        expect(id).toEqual(ASSIGNMENT_ID);
    });
});

test('create new assignment with no options', () => {
    const ASSIGNMENT_NAME = 'test assignment';

    const client = new Client({});
    const assignment = client.createNewAssignment(ASSIGNMENT_NAME);
    expect(assignment.options).toEqual({});
});

test('update assignment', () => {
    const ASSIGNMENT_ID = '568475923856028638773549';

    const http = {
        async put(url, body, unauth) {
            expect(url).toEqual(`/api/v1/assignment/${ASSIGNMENT_ID}`);
            expect(body).not.toBeInstanceOf(Assignment);
            expect(unauth).toBeFalsy();

            return { data: {} }
        }
    };

    const client = new Client(http);
    const assignment = new Assignment(ASSIGNMENT_ID, 'name!', { prop2: 'electric value' }, client);
    return client.updateAssignment(assignment).then(() => {
        expect(assignment.id).toEqual(ASSIGNMENT_ID);
    });
});

test('delete assignment', () => {
    const ASSIGNMENT_ID = '389549803457985430530893764095072467653';

    const http = {
        async delete(url, unauth) {
            expect(url).toEqual(`/api/v1/assignment/${ASSIGNMENT_ID}`);
            expect(unauth).toBeFalsy();

            return { data: {} };
        }
    };

    const client = new Client(http);
    return client.deleteAssignment(ASSIGNMENT_ID);
});

test('get settings', () => {
    const http = {
        async get(url, unauth) {
            expect(url).toEqual('/api/v1/settings');
            expect(unauth).toBeFalsy();

            return { data: SETTINGS_RESPONSE };
        }
    };

    const client = new Client(http);
    return client.getSettings().then(settings => {
        expect(settings['api port']).toEqual('8080');
    });
});

test('set settings', () => {
    const SETTINGS = {
        enabled: true,
        busy: 'yes',
        yesterday: 2,
    };

    expect.assertions(3);

    const http = {
        async put(url, body, unauth) {
            expect(url).toEqual('/api/v1/settings');
            expect(body).toEqual(SETTINGS);
            expect(unauth).toBeFalsy();
        }
    };

    const client = new Client(http);
    return client.setSettings(SETTINGS);
});

test('upload', () => {
    expect.hasAssertions();

    const aid = 'ASSIGNMENT-ID';
    const sid = 'SUBMISSION:ID';
    const file = { file: 'yes this is a file' };
    const http = {
        async post(url, body, unauth, type) {
            expect(url).toEqual(`/api/v1/assignment/${aid}/submission/direct-upload`);
            expect(body).toBe(file);
            expect(unauth).toBeFalsy();
            expect(type).not.toEqual('json');

            return { data: { id : sid } };
        }
    };

    const client = new Client(http);
    return client.upload(aid, file).then(s => expect(s.id).toEqual(sid));
});

test('get core logs', () => {
    expect.hasAssertions();

    const LOGS = [ 'ready', 'exiting...' ];

    const http = {
        async get(url, unauth, type) {
            expect(url).toEqual('/api/v1/logs/core');
            expect(unauth).toBeFalsy();

            return { data: { logs: LOGS } };
        }
    };

    const client = new Client(http);
    return client.getCoreLogs().then(l => expect(l).toEqual(LOGS));
});

test('get chart data', () => {
    const submission = {
        id: "SID",
        aid: "AID",
    };
    const metric = new Metric("name", 10);
    const entityLevel = "CLASS";
    expect.assertions(4);

    const http = {
        async post(url, data) {
            expect(url).toEqual(
                `/api/v1/assignment/${submission.aid}/submission/${submission.id}/benchmark/chartdata`);
            expect(data.metricNameString).toEqual(metric.name);
            expect(data.entityLevelString).toEqual(entityLevel);
            return { data: "hello" };
        }
    };

    const client = new Client(http);
    return client.getBenchmarkingChartData(submission, metric, entityLevel)
        .then(l => expect(l).toEqual("hello"));
});

test('get badge', () => {

    const sid = "sid";
    const aid = "aid";
    expect.assertions(2);

    const http = {
        async get(url, data) {
            expect(url).toEqual(
                `/api/v1/assignment/${aid}/submission/${sid}/benchmark/score`);
            return { data: {badge: "badgeData", errors: []} };
        }
    };

    const client = new Client(http);
    return client.getBadge(aid, sid)
        .then(l => expect(l).toEqual("badgeData"));
});
