import Course from "../src/Course"

test('create forwards to client', () => {
    const ID = 'ID!!!';
    const course = new Course(undefined, "name", "courseCode", 1980, [], [], [], undefined);

    expect.assertions(2);
    course.$client = {
        async addCourse(a) {
            expect(a).toBe(course);
            return ID;
        }
    };

    return course.create().then(id => expect(id).toEqual(ID));
});

test('update forwards to client', () => {
    const course = new Course("id2", "name", "courseCode", 1980, [], [], [], undefined);

    expect.assertions(1);
    course.$client = {
        async updateCourse(a) {
            expect(a).toBe(course);
        }
    };

    return course.update();
});

test('delete forwards to the client', () => {
    const ID = 'id 2';

    expect.assertions(1);
    const client = {
        async deleteCourse(cid) {
            expect(cid).toEqual(ID);
            return { data: { } };
        }
    };

    const course = new Course(ID, "name", "courseCode", 1980, [], [], [], client);
    return course.delete();
});

test('serialize does not contain taSet, instructorSet or assignmentIds', () => {
    const course = new Course(undefined, "name", "courseCode", 1980, [], [], [], undefined);
    const serialized = course.serialize();

    expect.assertions(3);
    expect(Object.keys(serialized)).not.toContain("taSet");
    expect(Object.keys(serialized)).not.toContain("instructorSet");
    expect(Object.keys(serialized)).not.toContain("assignmentIds");
});

test('addTA',() => {
    const ID = 'id 2';
    const USERNAME = 'bob';

    expect.assertions(2);
    const client = {
        async addUserToCourse(course, username, role) {
            expect(role).toEqual("ROLE_TA");
            expect(username).toEqual(USERNAME);
        }
    };

    const course = new Course(ID, "name", "courseCode", 1980, [], [], [], client);
    return course.addTA(USERNAME);
});

test('addTeacher',() => {
    const ID = 'id 2';
    const USERNAME = 'bob';

    expect.assertions(2);
    const client = {
        async addUserToCourse(course, username, role) {
            expect(role).toEqual("ROLE_TEACHER");
            expect(username).toEqual(USERNAME);
        }
    };

    const course = new Course(ID, "name", "courseCode", 1980, [], [], [], client);
    return course.addTeacher(USERNAME);
});

test('removeTeacher',() => {
    const ID = 'id 2';
    const USERNAME = 'bob';

    expect.assertions(2);
    const client = {
        async removeUserFromCourse(course, username, role) {
            expect(role).toEqual("ROLE_TEACHER");
            expect(username).toEqual(USERNAME);
        }
    };

    const course = new Course(ID, "name", "courseCode", 1980, [], [], [], client);
    return course.removeTeacher(USERNAME);
});

test('removeTa',() => {
    const ID = 'id 2';
    const USERNAME = 'bob';

    expect.assertions(2);
    const client = {
        async removeUserFromCourse(course, username, role) {
            expect(role).toEqual("ROLE_TA");
            expect(username).toEqual(USERNAME);
        }
    };

    const course = new Course(ID, "name", "courseCode", 1980, [], [], [], client);
    return course.removeTA(USERNAME);
});
