import Report from "../src/Report";
import Note from "../src/Note";
import Severity from "../src/Severity";

const TEST_REPORT = {"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[{"severity":"FAILURE","message":"Too complex: cyclomatic complexity 47 > 10"}]},"grade":null};
const TEST_REPORT_WITH_GRADE = {"name":"CYCLOMATIC_COMPLEXITY","notes":{"notes":[]},"grade":6.0};

test('parse', () => {
    const report = Report.parse(TEST_REPORT);

    expect(report.name).toEqual('CYCLOMATIC_COMPLEXITY');
    expect(report.notes).toHaveLength(1);
    expect(report.notes).toContainEqual(new Note(Severity.FAILURE, 'Too complex: cyclomatic complexity 47 > 10'));
    expect(report.grade).toBeNull();
});

test('filter by severity', () => {
    const failure = new Note(Severity.FAILURE, 'oh no');
    const tip = new Note(Severity.TIP, 'use your brain while programming');
    const report = new Report('TEST', [failure, tip], 2.0);

    expect(report.getNotesBySeverity(Severity.TIP)).toEqual([tip]);
});

test('parse with grade', () => {
    const report = Report.parse(TEST_REPORT_WITH_GRADE);
    expect(report.grade).toBe(6.0);
});

test('parse null', () => {
    const report = Report.parse(null);
    expect(report).toBeNull();
});
