import NodeHttp from '../../../src/backend/node/NodeHttp';
import Token from "../../../src/Token";

const http = new NodeHttp('https://jsonplaceholder.typicode.com/', {
    async getToken() {
        return new Token('token');
    }
});

test('GET some json', () => {
    return http.get('/posts/1', true)
        .then(res => {
            expect(res.data.id).toEqual(1)
        });
});

test('GET some json with token', () => {
    return http.get('/posts/1')
        .then(res => {
            expect(res.data.id).toEqual(1)
        });
});


test('GET some nonexistent json', () => {
    expect.assertions(1);
    return http.get('/posts/999999', true)
        .catch(res => {
            expect(res.status).toEqual(404);
        })
});

test('GET some nonexistent json', () => {
    expect.assertions(1);
    return http.get('/posts/999999')
        .catch(res => {
            expect(res.status).toEqual(404);
        })
});

test('POST some json', () => {
    return http.post('/posts', {title: 'test title', body: 'teeext', userId: 1}, true)
        .then(res => {
            expect(res.data.id).toEqual(101)
        });
});

test('PUT some json', () => {
    return http.put('/posts/5', {id: 1, title: 'new title', body: 'new body', userId: 1}, true)
        .then(res =>{
            expect(res.data.title).toEqual('new title')
        });
});

test('DELETE some json', () => {
    return http.delete('/users/1', true);
});

test('GET 403', () => {
    expect.assertions(2);
    const authProvider = {
        async getToken() { return new Token('tok'); },
        async handleInvalid() { expect(true).toBeTruthy(); }
    };
    const httpalt = new NodeHttp('https://httpstat.us/', authProvider);
    return httpalt.get('/403').catch(reason => expect(reason).not.toBeFalsy());
});
